@@ -557,6 +557,10 @@ class SyntheticContrastivePairGenerator:
                 return self._response_cache[cache_key]
         
         # Generate response
+        import time
+        start_time = time.time()
+        print(f"🟡 [Generation] Starting generation for prompt (first 100 chars): {prompt[:100]}...")
+        
         # Merge generation_kwargs with config, giving precedence to generation_kwargs
         merged_config = {**config, **self.generation_kwargs}
         
@@ -572,70 +576,51 @@ class SyntheticContrastivePairGenerator:
         else:
             formatted_prompt = self.model.format_prompt(prompt)
         
+        print(f"🟡 [Generation] Formatted prompt, tokenizing...")
         # Generate using the formatted prompt directly
-        # Debug logging
-        import threading
-        print(f"🔍 [Thread {threading.current_thread().name}] About to tokenize")
-        print(f"🔍 [Thread {threading.current_thread().name}] self.model type: {type(self.model)}")
-        print(f"🔍 [Thread {threading.current_thread().name}] self.model.tokenizer type: {type(self.model.tokenizer)}")
-        print(f"🔍 [Thread {threading.current_thread().name}] Is tokenizer callable: {callable(self.model.tokenizer)}")
-        
-        # Check tokenizer attributes
-        if hasattr(self.model.tokenizer, '__dict__'):
-            print(f"🔍 [Thread {threading.current_thread().name}] Tokenizer attributes: {list(self.model.tokenizer.__dict__.keys())[:10]}")
-        
-        # Check if tokenizer has the methods we need
-        print(f"🔍 [Thread {threading.current_thread().name}] Has __call__: {hasattr(self.model.tokenizer, '__call__')}")
-        print(f"🔍 [Thread {threading.current_thread().name}] Has pad_token_id: {hasattr(self.model.tokenizer, 'pad_token_id')}")
-        print(f"🔍 [Thread {threading.current_thread().name}] Has eos_token_id: {hasattr(self.model.tokenizer, 'eos_token_id')}")
-        
-        try:
-            # Try to get pad_token_id directly
-            if hasattr(self.model.tokenizer, 'pad_token_id'):
-                print(f"🔍 [Thread {threading.current_thread().name}] pad_token_id value: {self.model.tokenizer.pad_token_id}")
-        except Exception as e:
-            print(f"🔍 [Thread {threading.current_thread().name}] Error getting pad_token_id: {e}")
-        
         # Tokenize
-        print(f"🔍 [Thread {threading.current_thread().name}] Calling tokenizer now...")
-        try:
-            inputs = self.model.tokenizer(formatted_prompt, return_tensors="pt", padding=True, truncation=True)
-            print(f"🔍 [Thread {threading.current_thread().name}] Tokenization successful")
-        except Exception as e:
-            print(f"🔍 [Thread {threading.current_thread().name}] Tokenization failed with error: {e}")
-            print(f"🔍 [Thread {threading.current_thread().name}] Error type: {type(e)}")
-            import traceback
-            print(f"🔍 [Thread {threading.current_thread().name}] Full traceback:\n{traceback.format_exc()}")
-            raise
+        inputs = self.model.tokenizer(formatted_prompt, return_tensors="pt", padding=True, truncation=True)
+        print(f"🟡 [Generation] Tokenized, input shape: {inputs['input_ids'].shape}")
         
         inputs = {k: v.to(self.model.device) for k, v in inputs.items()}
         
+        print(f"🟡 [Generation] Starting model.generate() with max_new_tokens: {merged_config.get('max_new_tokens', 150)}")
+        gen_start = time.time()
+        
         # Generate
         with torch.no_grad():
-            # Get pad_token_id and eos_token_id safely
-            if hasattr(self.model.tokenizer, 'pad_token_id'):
-                pad_token_id = self.model.tokenizer.pad_token_id
-            elif isinstance(self.model.tokenizer, dict):
-                pad_token_id = self.model.tokenizer.get('pad_token_id', 0)
-            else:
-                pad_token_id = 0
-                
-            if hasattr(self.model.tokenizer, 'eos_token_id'):
-                eos_token_id = self.model.tokenizer.eos_token_id
-            elif isinstance(self.model.tokenizer, dict):
-                eos_token_id = self.model.tokenizer.get('eos_token_id', 0)
-            else:
-                eos_token_id = 0
+            # Import GenerationConfig from transformers
+            from transformers import GenerationConfig
             
-            outputs = self.model.hf_model.generate(
-                **inputs,
-                max_new_tokens=merged_config.get('max_new_tokens', 150),
-                temperature=merged_config.get('temperature', 0.7),
-                top_p=merged_config.get('top_p', 0.9),
-                do_sample=True,
-                pad_token_id=pad_token_id,
-                eos_token_id=eos_token_id,
-            )
+            # Create generation config with proper parameters
+            gen_config_dict = {
+                'max_new_tokens': merged_config.get('max_new_tokens', 150),
+                'temperature': merged_config.get('temperature', 0.7),
+                'top_p': merged_config.get('top_p', 0.9),
+                'do_sample': True,
+                'pad_token_id': self.model.tokenizer.pad_token_id,
+                'eos_token_id': self.model.tokenizer.eos_token_id,
+            }
+            
+            # Remove None values to avoid issues
+            gen_config_dict = {k: v for k, v in gen_config_dict.items() if v is not None}
+            
+            # Create GenerationConfig object
+            try:
+                generation_config = GenerationConfig(**gen_config_dict)
+                outputs = self.model.hf_model.generate(
+                    **inputs,
+                    generation_config=generation_config
+                )
+            except:
+                # Fallback for older transformers versions - use dict directly
+                outputs = self.model.hf_model.generate(
+                    **inputs,
+                    **gen_config_dict
+                )
+        
+        gen_time = time.time() - gen_start
+        print(f"🟡 [Generation] Generation completed in {gen_time:.2f} seconds")
         
         # Decode
         response = self.model.tokenizer.decode(outputs[0][inputs['input_ids'].shape[1]:], skip_special_tokens=True)
