@@ -7,6 +7,9 @@ This module provides ground truth evaluation using the lm-eval-harness framework
 import logging
 from typing import Any, Dict
 
+from wisent_guard.core.activations import ActivationAggregationStrategy, Activations
+from wisent_guard.core.layer import Layer
+
 logger = logging.getLogger(__name__)
 
 
@@ -203,10 +206,6 @@ class LMEvalHarnessGroundTruth:
             classification_results = []
             for response_data in generated_responses:
                 try:
-                    # Create layer object first
-                    from .activations import Activations
-                    from .layer import Layer
-
                     layer_obj = Layer(index=layer, type="transformer")
 
                     # Extract activations from generated response
@@ -214,7 +213,7 @@ class LMEvalHarnessGroundTruth:
                     activation_method = self._map_token_aggregation_to_activation_method(token_aggregation)
 
                     activation_obj = Activations(
-                        tensor=activation_tensor, layer=layer_obj, aggregation_method=activation_method
+                        tensor=activation_tensor, layer=layer_obj, aggregation_strategy=activation_method
                     )
 
                     # Get classifier prediction
@@ -349,9 +348,6 @@ class LMEvalHarnessGroundTruth:
 
                         # Extract activations from the text for classifier
                         try:
-                            from .activations import Activations
-                            from .layer import Layer
-
                             layer_obj = Layer(index=layer, type="transformer")
 
                             # Use a truncated version for activation extraction if text is too long
@@ -360,7 +356,7 @@ class LMEvalHarnessGroundTruth:
                             activation_method = self._map_token_aggregation_to_activation_method(token_aggregation)
 
                             activation_obj = Activations(
-                                tensor=activation_tensor, layer=layer_obj, aggregation_method=activation_method
+                                tensor=activation_tensor, layer=layer_obj, aggregation_strategy=activation_method
                             )
 
                             # Get classifier prediction (only if classifier is provided)
@@ -479,10 +475,6 @@ class LMEvalHarnessGroundTruth:
                         # Classify the best choice using the classifier
                         classification_score = None
                         try:
-                            # Create layer object first
-                            from .activations import Activations
-                            from .layer import Layer
-
                             layer_obj = Layer(index=layer, type="transformer")
 
                             # Extract activations from the best choice
@@ -490,7 +482,7 @@ class LMEvalHarnessGroundTruth:
                             activation_method = self._map_token_aggregation_to_activation_method(token_aggregation)
 
                             activation_obj = Activations(
-                                tensor=activation_tensor, layer=layer_obj, aggregation_method=activation_method
+                                tensor=activation_tensor, layer=layer_obj, aggregation_strategy=activation_method
                             )
 
                             # Get classifier prediction
@@ -665,16 +657,15 @@ class LMEvalHarnessGroundTruth:
 
     def _map_token_aggregation_to_activation_method(self, token_aggregation: str):
         """Map token aggregation string to activation method."""
-        from .activations import ActivationAggregationMethod
 
-        mapping = {
-            "average": ActivationAggregationMethod.MEAN,
-            "mean": ActivationAggregationMethod.MEAN,
-            "last": ActivationAggregationMethod.LAST_TOKEN,
-            "max": ActivationAggregationMethod.MAX,
+        mapping = {  # TODO This should be refactor, why we use strings as Token aggregation?
+            "average": ActivationAggregationStrategy.MEAN_POOLING,
+            "mean": ActivationAggregationStrategy.MEAN_POOLING,
+            "last": ActivationAggregationStrategy.LAST_TOKEN,
+            "max": ActivationAggregationStrategy.MAX_POOLING,
         }
 
-        return mapping.get(token_aggregation.lower(), ActivationAggregationMethod.MEAN)
+        return mapping.get(token_aggregation.lower(), ActivationAggregationStrategy.MEAN_POOLING)
 
     def _is_task_interface_task(self, task_name: str) -> bool:
         """Check if this is a TaskInterface task (not an lm-eval task)."""
@@ -1321,10 +1312,6 @@ class LMEvalHarnessGroundTruth:
             classification_results = []
             for i, code in enumerate(generated_codes):
                 try:
-                    # Create layer object
-                    from .activations import Activations
-                    from .layer import Layer
-
                     layer_obj = Layer(index=layer, type="transformer")
 
                     # Extract activations from generated code
@@ -1332,7 +1319,7 @@ class LMEvalHarnessGroundTruth:
                     activation_method = self._map_token_aggregation_to_activation_method(token_aggregation)
 
                     activation_obj = Activations(
-                        tensor=activation_tensor, layer=layer_obj, aggregation_method=activation_method
+                        tensor=activation_tensor, layer=layer_obj, aggregation_strategy=activation_method
                     )
 
                     # Get classifier prediction
