@@ -86,6 +86,12 @@ def setup_parser() -> argparse.ArgumentParser:
     )
     setup_multi_steer_parser(multi_steer_parser)
 
+    # Single-prompt evaluation command for real-time steering assessment
+    evaluate_parser = subparsers.add_parser(
+        "evaluate", help="Evaluate single prompt with steering vector and return quality scores"
+    )
+    setup_evaluate_parser(evaluate_parser)
+
     return parser
 
 
@@ -1557,6 +1563,22 @@ def setup_generate_vector_parser(parser):
     )
     parser.add_argument("--beta", type=float, default=1.0, help="Beta parameter for HPR method (default: 1.0)")
 
+    # Activation extraction configuration
+    parser.add_argument(
+        "--prompt-construction",
+        type=str,
+        default="multiple_choice",
+        choices=["multiple_choice", "role_playing", "direct_completion", "instruction_following"],
+        help="Strategy for constructing prompts from question-answer pairs (default: multiple_choice)",
+    )
+    parser.add_argument(
+        "--token-targeting",
+        type=str,
+        default="choice_token",
+        choices=["choice_token", "continuation_token", "last_token", "first_token", "mean_pooling", "max_pooling"],
+        help="Strategy for targeting tokens in activation extraction (default: choice_token)",
+    )
+
     # General options
     parser.add_argument("--verbose", action="store_true", help="Enable verbose output")
 
@@ -1596,3 +1618,42 @@ def setup_multi_steer_parser(parser):
         "--save-combined", type=str, default=None, help="Save the combined steering vector to this path"
     )
     parser.add_argument("--verbose", action="store_true", help="Enable verbose output showing weight calculations")
+
+
+def setup_evaluate_parser(parser):
+    """Set up the evaluate subcommand parser for single-prompt evaluation."""
+
+    # Required arguments
+    parser.add_argument("--vector", type=str, required=True, help="Path to steering vector file (.pt)")
+    parser.add_argument("--prompt", type=str, required=True, help="Prompt to evaluate")
+    parser.add_argument(
+        "--model", type=str, required=True, help="Model name or path (used for both generation and evaluation)"
+    )
+    parser.add_argument("--trait", type=str, required=True, help="Trait name (e.g., 'catholic', 'cynical')")
+
+    # Optional model configuration
+    parser.add_argument("--device", type=str, default=None, help="Device to run on (default: auto-detect)")
+
+    # Optional steering parameters
+    parser.add_argument(
+        "--steering-strength", type=float, default=2.0, help="Steering strength to apply (default: 2.0)"
+    )
+    parser.add_argument("--max-new-tokens", type=int, default=100, help="Maximum new tokens to generate (default: 100)")
+    parser.add_argument(
+        "--trait-description",
+        type=str,
+        default=None,
+        help="Optional description of the trait (default: use trait name)",
+    )
+
+    # Optional threshold parameters
+    parser.add_argument(
+        "--trait-threshold", type=float, default=None, help="Minimum trait quality threshold (-1 to 1 scale)"
+    )
+    parser.add_argument(
+        "--answer-threshold", type=float, default=None, help="Minimum answer quality threshold (0 to 1 scale)"
+    )
+
+    # Output options
+    parser.add_argument("--verbose", action="store_true", help="Enable verbose output")
+    parser.add_argument("--json", action="store_true", help="Output results as JSON")
